;; compute reflectance and transmittance of all diffracted orders
;; for a planewave incident on a slanted grating with design based on
;; T. Levola and P. Laakonen, Optics Express, Vol. 15, pp. 2067-74, 2007

(set-param! resolution 50)  ; pixels/μm

(define-param dpml 1.0)     ; PML length
(define-param dair 4.0)     ; padding length between PML and grating
(define-param dsub 3.0)     ; substrate thickness
(define-param d 3.5)        ; grating period
(define-param h 1.9)        ; grating height
(define-param g 0.7)        ; grating duty cycle
(define-param theta-1 12.8) ; grating sidewall angle #1
(define-param theta-2 27.4) ; grating sidewall angle #2

(set! theta-1 (deg->rad theta-1))
(set! theta-2 (deg->rad theta-2))

(define sx (+ dpml dair h dsub dpml))
(define sy d)

(define cell (make lattice (size sx sy no-size)))
(set! geometry-lattice cell)

(define boundary-layers (list (make absorber (thickness dpml) (direction X))))
(set! pml-layers boundary-layers)

(define-param wvl 0.5)       ; center wavelength
(define fcen (/ wvl))        ; center frequency
(define df (* 0.05 fcen))    ; frequency width

(define-param src-pol 1)     ; source polarization (1: Ez, 2: Hz, default: Ez)
(define src-cmpt (if (= src-pol 1) Ez Hz))
(define eparity (if (= src-pol 1) ODD-Z EVEN-Z))

; rotation angle (degrees) of incident planewave source; counter clockwise (CCW) about Z axis, 0 degrees along +X axis
(define-param src-angle 0)
(set! src-angle (deg->rad src-angle))

(define k (rotate-vector3 (vector3 0 0 1) src-angle (vector3 fcen 0 0)))
(set! k-point k)

(define (pw-amp k x0)
  (lambda (x)
    (exp (* 0+1i 2 pi (vector3-dot k (vector3- x x0))))))

(define src-pt (vector3 (+ (* -0.5 sx) dpml (* 0.2 dair)) 0 0))
(define pulse-src (list (make source
                          (src (make gaussian-src (frequency fcen) (fwidth df)))
                          (component src-cmpt)
                          (center (+ (* -0.5 sx) dpml (* 0.5 dsub)) 0 0)
                          (size 0 sy 0)
                          (amp-func (pw-amp k src-pt)))))

(set! sources pulse-src)

(define refl-pt (vector3 (+ (* -0.5 sx) dpml (* 0.7 dair)) 0 0))
(define src-flux (add-flux fcen 0 1 (make flux-region (center refl-pt) (size 0 sy 0))))

(run-sources+ 100)

(save-flux "flux" src-flux)
(define input-flux (get-fluxes src-flux))

(reset-meep)

(set! geometry-lattice cell)

(set! pml-layers boundary-layers)

(set! sources pulse-src)

(set! k-point k)

(define ng 1.716)            ; episulfide refractive index @ 0.532 μm
(define glass (make medium (index ng)))

(set! geometry (list (make block
                       (material glass)
                       (size (+ dpml dsub) infinity infinity)
                       (center (- (* 0.5 sx) (* 0.5 (+ dpml dsub))) 0 0))
                     (make prism
                       (material glass)
                       (vertices (list
                                  (vector3 (- (* 0.5 sx) dpml dsub) (* 0.5 sy) 0)
                                  (vector3 (- (* 0.5 sx) dpml dsub h) (- (* 0.5 sy) (* h (tan theta-2))) 0)
                                  (vector3 (- (* 0.5 sx) dpml dsub h) (+ (* -0.5 sy) g (* (- h) (tan theta-1))) 0)
                                  (vector3 (- (* 0.5 sx) dpml dsub) (+ (* -0.5 sy) g))))
                       (axis 0 0 1)
                       (center auto-center)
                       (height infinity))))

(define refl-flux (add-flux fcen 0 1 (make flux-region (center refl-pt) (size 0 sy 0))))
(load-minus-flux "flux" refl-flux)

(define tran-pt (vector3 (- (* 0.5 sx) dpml (* 0.5 dsub)) 0 0))
(define tran-flux (add-flux fcen 0 1 (make flux-region (center tran-pt) (size 0 sy 0))))

(run-sources+ 500) ; (at-beginning output-epsilon))

(define kdom-tol 1e-2)
(define angle-tol 1e-6)

(define modulo-float (lambda (n1 n2) (- n1 (* (floor (/ n1 n2)) n2))))

(define res (get-eigenmode-coefficients refl-flux (list 1) #:eig-parity eparity))
(define coeffs (list-ref res 0))
(define kdom (list-ref res 3))
(define Rsum 0)
(define r-angle 0)
(define Tsum 0)
(define t-angle 0)

(if (= src-angle 0)
    (let
        ((nm-r (* 0.5 (- (floor (* (- fcen (vector3-y k)) d)) (ceiling (* (- (- fcen) (vector3-y k)) d)))))
         (nm-t (* 0.5 (- (floor (* (- (* fcen ng) (vector3-y k)) d)) (ceiling (* (- (- (* fcen ng)) (vector3-y k)) d))))))
      (set! res (get-eigenmode-coefficients refl-flux (arith-sequence 1 1 nm-r) #:eig-parity (+ eparity EVEN-Y)))
      (set! coeffs (list-ref res 0))
      (set! kdom (list-ref res 3))
      (map (lambda (nm)
             (let ((r-kdom (list-ref kdom nm))
                   (Rmode (/ (sqr (magnitude (array-ref coeffs nm 0 1))) (list-ref input-flux 0))))
               (if (> (vector3-x r-kdom) kdom-tol)
                   (begin
                     (set! r-angle (if (> (modulo-float (vector3-x r-kdom) fcen) angle-tol) (* (if (positive? (vector3-y r-kdom)) +1 -1) (acos (/ (vector3-x r-kdom) fcen))) 0))
                     (print "refl: (even-y), " nm ", " (rad->deg r-angle) ", " Rmode "\n")
                     (set! Rsum (+ Rsum Rmode))))))
           (arith-sequence 0 1 nm-r))

      (set! res (get-eigenmode-coefficients refl-flux (arith-sequence 1 1 nm-r) #:eig-parity (+ eparity ODD-Y)))
      (set! coeffs (list-ref res 0))
      (set! kdom (list-ref res 3))
      (map (lambda (nm)
             (let ((r-kdom (list-ref kdom nm))
                   (Rmode (/ (sqr (magnitude (array-ref coeffs nm 0 1))) (list-ref input-flux 0))))
               (if (> (vector3-x r-kdom) kdom-tol)
                   (begin
                     (set! r-angle (if (> (modulo-float (vector3-x r-kdom) fcen) angle-tol) (* (if (positive? (vector3-y r-kdom)) +1 -1) (acos (/ (vector3-x r-kdom) fcen))) 0))
                     (print "refl: (odd-y), " nm ", " (rad->deg r-angle) ", " Rmode "\n")
                     (set! Rsum (+ Rsum Rmode))))))
           (arith-sequence 0 1 nm-r))

      (set! res (get-eigenmode-coefficients tran-flux (arith-sequence 1 1 nm-t) #:eig-parity (+ eparity EVEN-Y)))
      (set! coeffs (list-ref res 0))
      (set! kdom (list-ref res 3))
      (map (lambda (nm)
             (let ((t-kdom (list-ref kdom nm))
                   (Tmode (/ (sqr (magnitude (array-ref coeffs nm 0 0))) (list-ref input-flux 0))))
               (if (> (vector3-x t-kdom) kdom-tol)
                   (begin
                     (set! t-angle (if (> (modulo-float (vector3-x t-kdom) (* fcen ng)) angle-tol) (* (if (positive? (vector3-y t-kdom)) +1 -1) (acos (/ (vector3-x t-kdom) (* fcen ng)))) 0))
                     (print "tran: (even-y), " nm ", " (rad->deg t-angle) ", " Tmode "\n")
                     (set! Tsum (+ Tsum Tmode))))))
           (arith-sequence 0 1 nm-t))

      (set! res (get-eigenmode-coefficients tran-flux (arith-sequence 1 1 nm-t) #:eig-parity (+ eparity ODD-Y)))
      (set! coeffs (list-ref res 0))
      (set! kdom (list-ref res 3))
      (map (lambda (nm)
             (let ((t-kdom (list-ref kdom nm))
                   (Tmode (/ (sqr (magnitude (array-ref coeffs nm 0 0))) (list-ref input-flux 0))))
               (if (> (vector3-x t-kdom) kdom-tol)
                   (begin
                     (set! t-angle (if (> (modulo-float (vector3-x t-kdom) (* fcen ng)) angle-tol) (* (if (positive? (vector3-y t-kdom)) +1 -1) (acos (/ (vector3-x t-kdom) (* fcen ng)))) 0))
                     (print "tran: (odd-y), " nm ", " (rad->deg t-angle) ", " Tmode "\n")
                     (set! Tsum (+ Tsum Tmode))))))
           (arith-sequence 0 1 nm-t)))
    (let
        ((nm-r (- (floor (* (- fcen (vector3-y k)) d)) (ceiling (* (- (- fcen) (vector3-y k)) d))))
         (nm-t (- (floor (* (- (* fcen ng) (vector3-y k)) d)) (ceiling (* (- (- (* fcen ng)) (vector3-y k)) d)))))
      (set! res (get-eigenmode-coefficients refl-flux (arith-sequence 1 1 nm-r) #:eig-parity eparity))
      (set! coeffs (list-ref res 0))
      (set! kdom (list-ref res 3))
      (map (lambda (nm)
             (let ((r-kdom (list-ref kdom nm))
                   (Rmode (/ (sqr (magnitude (array-ref coeffs nm 0 1))) (list-ref input-flux 0))))
               (if (> (vector3-x r-kdom) kdom-tol)
                   (begin
                     (set! r-angle (if (> (modulo-float (vector3-x r-kdom) fcen) angle-tol) (* (if (positive? (vector3-y r-kdom)) +1 -1) (acos (/ (vector3-x r-kdom) fcen))) 0))
                     (print "refl:, " nm ", " (rad->deg r-angle) ", " Rmode "\n")
                     (set! Rsum (+ Rsum Rmode))))))
           (arith-sequence 0 1 nm-r))

      (set! res (get-eigenmode-coefficients tran-flux (arith-sequence 1 1 nm-t) #:eig-parity eparity))
      (set! coeffs (list-ref res 0))
      (set! kdom (list-ref res 3))
      (map (lambda (nm)
             (let ((t-kdom (list-ref kdom nm))
                   (Tmode (/ (sqr (magnitude (array-ref coeffs nm 0 0))) (list-ref input-flux 0))))
               (if (> (vector3-x t-kdom) kdom-tol)
                   (begin
                     (set! t-angle (if (> (modulo-float (vector3-x t-kdom) (* fcen ng)) angle-tol) (* (if (positive? (vector3-y t-kdom)) +1 -1) (acos (/ (vector3-x t-kdom) (* fcen ng)))) 0))
                     (print "tran:, " nm ", " (rad->deg t-angle) ", " Tmode "\n")
                     (set! Tsum (+ Tsum Tmode))))))
           (arith-sequence 0 1 nm-t))))

(print "total:, " Rsum ", " Tsum ", " (+ Rsum Tsum) "\n")